@@ -1,6 +1,7 @@
 /* imlib.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -34,6 +35,10 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <arpa/inet.h>
 #include <netdb.h>
 
+#ifdef HAVE_LIBCURL
+#include <curl/curl.h>
+#endif
+
 Display *disp = NULL;
 Visual *vis = NULL;
 Screen *scr = NULL;
@@ -160,7 +165,7 @@ int feh_load_image(Imlib_Image * im, feh_file * file)
 
 	if ((err) || (!im)) {
 		if (opt.verbose && !opt.quiet) {
-			fprintf(stdout, "\n");
+			fputs("\n", stdout);
 			reset_output = 1;
 		}
 		/* Check error code */
@@ -231,8 +236,16 @@ int feh_load_image(Imlib_Image * im, feh_file * file)
 	return(1);
 }
 
+#ifdef HAVE_LIBCURL
+
 char *feh_http_load_image(char *url)
 {
+	CURL *curl;
+	CURLcode res;
+	char *sfn;
+	FILE *sfp;
+	int fd = -1;
+	char *ebuff;
 	char *tmpname;
 	char *basename;
 	char *path = NULL;
@@ -248,263 +261,63 @@ char *feh_http_load_image(char *url)
 	basename = strrchr(url, '/') + 1;
 	tmpname = feh_unique_filename(path, basename);
 
-	if (opt.builtin_http) {
-		/* state for HTTP header parser */
-#define SAW_NONE    1
-#define SAW_ONE_CM  2
-#define SAW_ONE_CJ  3
-#define SAW_TWO_CM  4
-#define IN_BODY     5
-
-#define OUR_BUF_SIZE 1024
-#define EOL "\015\012"
-
-		int sockno = 0;
-		int size;
-		int body = SAW_NONE;
-		struct sockaddr_in addr;
-		struct hostent *hptr;
-		char *hostname;
-		char *get_string;
-		char *host_string;
-		char *query_string;
-		char *get_url;
-		static char buf[OUR_BUF_SIZE];
-		char ua_string[] = "User-Agent: feh image viewer";
-		char accept_string[] = "Accept: image/*";
-		FILE *fp;
-
-		D(("using builtin http collection\n"));
-		fp = fopen(tmpname, "w");
-		if (!fp) {
-			weprintf("couldn't write to file %s:", tmpname);
-			free(tmpname);
-			return(NULL);
-		}
-
-		hostname = feh_strip_hostname(url);
-		if (!hostname) {
-			weprintf("couldn't work out hostname from %s:", url);
-			fclose(fp);
-			unlink(tmpname);
-			free(tmpname);
-			return(NULL);
-		}
-
-		D(("trying hostname %s\n", hostname));
-
-		if (!(hptr = feh_gethostbyname(hostname))) {
-			weprintf("error resolving host %s:", hostname);
-			fclose(fp);
-			unlink(tmpname);
-			free(hostname);
-			free(tmpname);
-			return(NULL);
-		}
-
-		/* Copy the address of the host to socket description. */
-		memcpy(&addr.sin_addr, hptr->h_addr, hptr->h_length);
-
-		/* Set port and protocol */
-		addr.sin_family = AF_INET;
-		addr.sin_port = htons(80);
-
-		if ((sockno = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
-			weprintf("error opening socket:");
-			fclose(fp);
-			unlink(tmpname);
-			free(tmpname);
-			free(hostname);
-			return(NULL);
-		}
-		if (connect(sockno, (struct sockaddr *) &addr, sizeof(addr)) == -1) {
-			weprintf("error connecting socket:");
-			fclose(fp);
-			unlink(tmpname);
-			free(tmpname);
-			free(hostname);
-			return(NULL);
-		}
-
-		get_url = strchr(url, '/') + 2;
-		get_url = strchr(get_url, '/');
-
-		get_string = estrjoin(" ", "GET", get_url, "HTTP/1.0", NULL);
-		host_string = estrjoin(" ", "Host:", hostname, NULL);
-		query_string = estrjoin(EOL, get_string, host_string, accept_string, ua_string, "", "", NULL);
-		/* At this point query_string looks something like
-		 **
-		 **    GET /dir/foo.jpg?123456 HTTP/1.0^M^J
-		 **    Host: www.example.com^M^J
-		 **    Accept: image/ *^M^J
-		 **    User-Agent: feh image viewer^M^J
-		 **    ^M^J
-		 **
-		 ** Host: is required by HTTP/1.1 and very important for some sites,
-		 ** even with HTTP/1.0
-		 **
-		 ** -- BEG
-		 */
-		if ((send(sockno, query_string, strlen(query_string), 0)) == -1) {
-			free(get_string);
-			free(host_string);
-			free(query_string);
-			free(tmpname);
-			free(hostname);
-			weprintf("error sending over socket:");
-			return(NULL);
-		}
-		free(get_string);
-		free(host_string);
-		free(query_string);
-		free(hostname);
-
-		while ((size = read(sockno, &buf, OUR_BUF_SIZE))) {
-			if (body == IN_BODY) {
-				fwrite(buf, 1, size, fp);
-			} else {
-				int i;
-
-				for (i = 0; i < size; i++) {
-					/* We are looking for ^M^J^M^J, but will accept
-					 ** ^J^J from broken servers. Stray ^Ms will be
-					 ** ignored.
-					 **
-					 ** TODO:
-					 ** Checking the headers for a
-					 **    Content-Type: image/ *
-					 ** header would help detect problems with results.
-					 ** Maybe look at the response code too? But there is
-					 ** no fundamental reason why a 4xx or 5xx response
-					 ** could not return an image, it is just the 3xx
-					 ** series we need to worry about.
-					 **
-					 ** Also, grabbing the size from the Content-Length
-					 ** header and killing the connection after that
-					 ** many bytes where read would speed up closing the
-					 ** socket.
-					 ** -- BEG
-					 */
-
-					switch (body) {
-
-					case IN_BODY:
-						fwrite(buf + i, 1, size - i, fp);
-						i = size;
-						break;
-
-					case SAW_ONE_CM:
-						if (buf[i] == '\012') {
-							body = SAW_ONE_CJ;
-						} else {
-							body = SAW_NONE;
-						}
-						break;
-
-					case SAW_ONE_CJ:
-						if (buf[i] == '\015') {
-							body = SAW_TWO_CM;
-						} else {
-							if (buf[i] == '\012') {
-								body = IN_BODY;
-							} else {
-								body = SAW_NONE;
-							}
-						}
-						break;
-
-					case SAW_TWO_CM:
-						if (buf[i] == '\012') {
-							body = IN_BODY;
-						} else {
-							body = SAW_NONE;
-						}
-						break;
-
-					case SAW_NONE:
-						if (buf[i] == '\015') {
-							body = SAW_ONE_CM;
-						} else {
-							if (buf[i] == '\012') {
-								body = SAW_ONE_CJ;
-							}
-						}
-						break;
-
-					}	/* switch */
-				}	/* for i */
-			}
-		}		/* while read */
-		close(sockno);
-		fclose(fp);
-	} else {
-		int pid;
-		int status;
-
-		if ((pid = fork()) < 0) {
-			weprintf("open url: fork failed:");
-			free(tmpname);
-			return(NULL);
-		} else if (pid == 0) {
-			char *quiet = NULL;
-
-			if (!opt.verbose)
-				quiet = estrdup("-q");
-
-			execlp("wget", "wget", "--cache=off", "-O", tmpname, url, quiet, NULL);
-			eprintf("url: Is 'wget' installed? Failed to exec wget:");
-		} else {
-			waitpid(pid, &status, 0);
-
-			if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
-				weprintf("url: wget failed to load URL %s\n", url);
-				unlink(tmpname);
-				free(tmpname);
-				return(NULL);
-			}
-		}
+	curl = curl_easy_init();
+	if (!curl) {
+		weprintf("open url: libcurl initialization failure");
+		return NULL;
 	}
+	
+	sfn = estrjoin("_", tmpname, "XXXXXX", NULL);
+	free(tmpname);
+	fd = mkstemp(sfn);
+	if (fd != -1) {
+		sfp = fdopen(fd, "w+");
+		if (sfp != NULL) {
+			curl_easy_setopt(curl, CURLOPT_URL, url);
+			curl_easy_setopt(curl, CURLOPT_WRITEDATA, sfp);
+			ebuff = emalloc(CURL_ERROR_SIZE);
+			curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, ebuff);
+			curl_easy_setopt(curl, CURLOPT_FAILONERROR, 1L);
 
-	return(tmpname);
+			res = curl_easy_perform(curl);
+			curl_easy_cleanup(curl);
+			if (res != CURLE_OK) {
+				weprintf("open url: %s", ebuff);
+				unlink(sfn);
+				close(fd);
+				free(sfn);
+				sfn = NULL;
+			}
+
+			free(ebuff);
+			fclose(sfp);
+			return sfn;
+		} else {
+			weprintf("open url: fdopen failed:");
+			unlink(sfn);
+			free(sfn);
+			close(fd);
+		}
+	} else {
+		weprintf("open url: mkstemp failed:");
+		free(sfn);
+	}
+	curl_easy_cleanup(curl);
+	return NULL;
 }
 
-struct hostent *feh_gethostbyname(const char *name)
+#else				/* HAVE_LIBCURL */
+
+char *feh_http_load_image(char *url)
 {
-	struct hostent *hp;
-	unsigned long addr;
-
-	addr = (unsigned long) inet_addr(name);
-	if ((int) addr != -1)
-		hp = gethostbyaddr((char *) &addr, sizeof(addr), AF_INET);
-	else
-		hp = gethostbyname(name);
-	return(hp);
+	weprintf(
+		"Cannot load image %s\n Please recompile with libcurl support",
+		url
+	);
+	return NULL;
 }
 
-char *feh_strip_hostname(char *url)
-{
-	char *ret;
-	char *start;
-	char *finish;
-	int len;
-
-	start = strchr(url, '/');
-	if (!start)
-		return(NULL);
-
-	start += 2;
-
-	finish = strchr(start, '/');
-	if (!finish)
-		return(NULL);
-
-	len = finish - start;
-
-	ret = emalloc(len + 1);
-	strncpy(ret, start, len);
-	ret[len] = '\0';
-	return(ret);
-}
+#endif				/* HAVE_LIBCURL */
 
 void feh_draw_zoom(winwidget w)
 {
@@ -554,12 +367,76 @@ void feh_draw_zoom(winwidget w)
 	return;
 }
 
-void feh_draw_filename(winwidget w)
+void im_weprintf(winwidget w, char *fmt, ...)
+{
+	va_list args;
+	char *errstr = emalloc(1024);
+
+	fflush(stdout);
+	fputs(PACKAGE " WARNING: ", stderr);
+
+	va_start(args, fmt);
+	vsnprintf(errstr, 1024, fmt, args);
+	va_end(args);
+
+	if (w)
+		w->errstr = errstr;
+
+	fputs(errstr, stderr);
+	if (fmt[0] != '\0' && fmt[strlen(fmt) - 1] == ':')
+		fprintf(stderr, " %s", strerror(errno));
+	fputs("\n", stderr);
+}
+
+
+void feh_draw_errstr(winwidget w)
 {
 	static Imlib_Font fn = NULL;
 	int tw = 0, th = 0;
 	Imlib_Image im = NULL;
 	static DATA8 atab[256];
+
+	if (!w->im)
+		return;
+
+	if (opt.font)
+		fn = gib_imlib_load_font(opt.font);
+
+	if (!fn)
+		fn = gib_imlib_load_font(DEFAULT_FONT);
+
+	if (!fn)
+		eprintf("Unable to draw error message. Dying to be safe.");
+
+	memset(atab, 0, sizeof(atab));
+
+	/* Work out how high the font is */
+	gib_imlib_get_text_size(fn, w->errstr, NULL, &tw, &th, IMLIB_TEXT_TO_RIGHT);
+
+	tw += 3;
+	th += 3;
+	im = imlib_create_image(tw, th);
+	if (!im)
+		eprintf("Couldn't create errstr image. Out of memory?");
+
+	gib_imlib_image_set_has_alpha(im, 1);
+	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th, NULL, NULL, NULL, atab);
+	gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
+
+	gib_imlib_text_draw(im, fn, NULL, 2, 2, w->errstr, IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
+	gib_imlib_text_draw(im, fn, NULL, 1, 1, w->errstr, IMLIB_TEXT_TO_RIGHT, 255, 0, 0, 255);
+	free(w->errstr);
+	w->errstr = NULL;
+	gib_imlib_render_image_on_drawable(w->bg_pmap, im, 0, w->h - th, 1, 1, 0);
+	gib_imlib_free_image_and_decache(im);
+}
+
+void feh_draw_filename(winwidget w)
+{
+	static Imlib_Font fn = NULL;
+	int tw = 0, th = 0, nw = 0;
+	Imlib_Image im = NULL;
+	static DATA8 atab[256];
 	char *s = NULL;
 	int len = 0;
 
@@ -578,17 +455,29 @@ void feh_draw_filename(winwidget w)
 	}
 
 	if (!fn) {
-		weprintf("Couldn't load font for filename printing");
+		eprintf("Couldn't load font for filename printing");
 		return;
 	}
 
 	memset(atab, 0, sizeof(atab));
 
 	/* Work out how high the font is */
-	gib_imlib_get_text_size(fn, FEH_FILE(w->file->data)->filename, NULL, &tw, &th, IMLIB_TEXT_TO_RIGHT);
+	gib_imlib_get_text_size(fn, FEH_FILE(w->file->data)->filename, NULL, &tw,
+			&th, IMLIB_TEXT_TO_RIGHT);
+
+	if (gib_list_length(filelist) > 1) {
+		len = snprintf(NULL, 0, "%d of %d",  gib_list_length(filelist),
+				gib_list_length(filelist)) + 1;
+		s = emalloc(len);
+		snprintf(s, len, "%d of %d", gib_list_num(filelist, current_file) +
+				1, gib_list_length(filelist));
+
+		gib_imlib_get_text_size(fn, s, NULL, &nw, NULL, IMLIB_TEXT_TO_RIGHT);
+
+		if (nw > tw)
+			tw = nw;
+	}
 
-	/* tw is no longer correct, if the filename is shorter than
-	 * the string "%d of %d" used below in fullscreen mode */
 	tw += 3;
 	th += 3;
 	im = imlib_create_image(tw, 2 * th);
@@ -596,20 +485,16 @@ void feh_draw_filename(winwidget w)
 		eprintf("Couldn't create image. Out of memory?");
 
 	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, 2 * th, NULL, NULL, NULL, atab);
+	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, 2 * th, NULL,
+			NULL, NULL, atab);
 	gib_imlib_image_fill_rectangle(im, 0, 0, tw, 2 * th, 0, 0, 0, 0);
 
 	gib_imlib_text_draw(im, fn, NULL, 2, 2, FEH_FILE(w->file->data)->filename,
 			IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
 	gib_imlib_text_draw(im, fn, NULL, 1, 1, FEH_FILE(w->file->data)->filename,
 			IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
-	/* Print the position in the filelist, if we have >=2 files */
-	if (gib_list_length(filelist) > 1) {
-		/* sic! */
-		len = snprintf(NULL, 0, "%d of %d", gib_list_length(filelist), gib_list_length(filelist)) + 1;
-		s = emalloc(len);
-		snprintf(s, len, "%d of %d", gib_list_num(filelist, current_file) + 1, gib_list_length(filelist));
-		/* This should somehow be right-aligned */
+
+	if (s) {
 		gib_imlib_text_draw(im, fn, NULL, 2, th + 1, s, IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
 		gib_imlib_text_draw(im, fn, NULL, 1, th, s, IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
 		free(s);
@@ -647,7 +532,7 @@ void feh_draw_info(winwidget w)
 	}
 
 	if (!fn) {
-		weprintf("Couldn't load font for filename printing");
+		eprintf("Couldn't load font for filename printing");
 		return;
 	}
 
@@ -695,7 +580,7 @@ void feh_draw_info(winwidget w)
 	return;
 }
 
-char *build_caption_filename(feh_file * file)
+char *build_caption_filename(feh_file * file, short create_dir)
 {
 	char *caption_filename;
 	char *s, *dir, *caption_dir;
@@ -714,6 +599,8 @@ char *build_caption_filename(feh_file * file)
 	D(("dir %s, cp %s, cdir %s\n", dir, opt.caption_path, caption_dir))
 
 	if (stat(caption_dir, &cdir_stat) == -1) {
+		if (!create_dir)
+			return NULL;
 		if (mkdir(caption_dir, 0755) == -1)
 			eprintf("Failed to create caption directory %s:", caption_dir);
 	} else if (!S_ISDIR(cdir_stat.st_mode))
@@ -749,9 +636,12 @@ void feh_draw_caption(winwidget w)
 
 	if (!file->caption) {
 		char *caption_filename;
-		caption_filename = build_caption_filename(file);
-		/* read caption from file */
-		file->caption = ereadfile(caption_filename);
+		caption_filename = build_caption_filename(file, 0);
+		if (caption_filename)
+			/* read caption from file */
+			file->caption = ereadfile(caption_filename);
+		else
+			file->caption = estrdup("");
 		free(caption_filename);
 	}
 
@@ -787,7 +677,7 @@ void feh_draw_caption(winwidget w)
 	}
 
 	if (!fn) {
-		weprintf("Couldn't load font for caption printing");
+		eprintf("Couldn't load font for caption printing");
 		return;
 	}
 
@@ -874,7 +764,7 @@ void feh_display_status(char stat)
 		if (reset_output) {
 			/* There's just been an error message. Unfortunate ;) */
 			for (j = 0; j < (((i % 50) + ((i % 50) / 10)) + 7); j++)
-				fprintf(stdout, " ");
+				putc(' ', stdout);
 		}
 
 		if (!(i % 50)) {
@@ -884,11 +774,11 @@ void feh_display_status(char stat)
 					i, init_len, len, ((int) ((float) i / init_len * 100)));
 
 		} else if ((!(i % 10)) && (!reset_output))
-			fprintf(stdout, " ");
+			putc(' ', stdout);
 
 		reset_output = 0;
 	} else
-		fprintf(stdout, "[  0%%] ");
+		fputs("[  0%] ", stdout);
 
 	fprintf(stdout, "%c", stat);
 	fflush(stdout);
@@ -916,7 +806,7 @@ void feh_edit_inplace_orient(winwidget w, int orientation)
 		gib_imlib_free_image(old);
 		feh_reload_image(w, 1, 1);
 	} else {
-		weprintf("failed to load image from disk to edit it in place\n");
+		im_weprintf(w, "failed to load image from disk to edit it in place");
 	}
 
 	return;
@@ -1033,21 +923,23 @@ void feh_edit_inplace_lossless_rotate(winwidget w, int orientation)
 	snprintf(file_str, len, "%s", filename);
 
 	if ((pid = fork()) < 0) {
-		weprintf("lossless rotate: fork failed:");
+		im_weprintf(w, "lossless rotate: fork failed:");
 		return;
 	} else if (pid == 0) {
 
 		execlp("jpegtran", "jpegtran", "-copy", "all", "-rotate",
 				rotate_str, "-outfile", file_str, file_str, NULL);
 
-		eprintf("lossless rotate: Is 'jpegtran' installed? Failed to exec:");
+		im_weprintf(w, "lossless rotate: Is 'jpegtran' installed? Failed to exec:");
+		return;
 	} else {
 		waitpid(pid, &status, 0);
 
 		if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
-			weprintf("lossless rotate: Got exitcode %d from jpegtran."
-					" Commandline was:\n"
-					"jpegtran -copy all -rotate %d -outfile %s %s\n",
+			im_weprintf(w,
+					"lossless rotate: Got exitcode %d from jpegtran."
+					" Commandline was: "
+					"jpegtran -copy all -rotate %s -outfile %s %s",
 					status >> 8, rotate_str, file_str, file_str);
 			return;
 		}
@@ -1066,7 +958,7 @@ void feh_draw_actions(winwidget w)
 	int i = 0;
 	int num_actions = 0;
 	int cur_action = 0;
-	char index[1];
+	char index[2];
 	char *line;
 
 	/* Count number of defined actions. This method sucks a bit since it needs
@@ -1096,7 +988,7 @@ void feh_draw_actions(winwidget w)
 	}
 
 	if (!fn) {
-		weprintf("Couldn't load font for actions printing");
+		eprintf("Couldn't load font for actions printing");
 		return;
 	}
 
