@@ -1,6 +1,7 @@
 /* main.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -43,8 +44,10 @@ int main(int argc, char **argv)
 
 	init_imlib_fonts();
 
-	if (opt.display)
+	if (opt.display) {
 		init_x_and_imlib();
+		init_keyevents();
+	}
 
 	feh_event_init();
 
