@@ -1,6 +1,7 @@
 /* options.h
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -26,6 +27,9 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #ifndef OPTIONS_H
 #define OPTIONS_H
 
+#define ZOOM_MODE_FILL 1
+#define ZOOM_MODE_MAX  2
+
 struct __fehoptions {
 	unsigned char multiwindow;
 	unsigned char montage;
@@ -50,7 +54,6 @@ struct __fehoptions {
 	unsigned char randomize;
 	unsigned char jump_on_resort;
 	unsigned char full_screen;
-	unsigned char auto_zoom;
 	unsigned char draw_filename;
 	unsigned char list;
 	unsigned char quiet;
@@ -60,12 +63,12 @@ struct __fehoptions {
 	unsigned char reverse;
 	unsigned char no_menus;
 	unsigned char scale_down;
-	unsigned char builtin_http;
 	unsigned char bgmode;
 	unsigned char xinerama;
 	unsigned char screen_clip;
 	unsigned char hide_pointer;
 	unsigned char draw_actions;
+	unsigned char draw_info;
 	unsigned char cache_thumbnails;
 	unsigned char cycle_once;
 	unsigned char hold_actions[10];
@@ -84,7 +87,6 @@ struct __fehoptions {
 	char *customlist;
 	char *menu_bg;
 	char *image_bg;
-	char *rcfile;
 	char *menu_style;
 	char *caption_path;
 	char *start_list_at;
@@ -106,6 +108,7 @@ struct __fehoptions {
 	unsigned char no_blur_ctrl_mask;
 	unsigned char no_pan_ctrl_mask;
 
+	int force_aliasing;
 	int thumb_w;
 	int thumb_h;
 	int limit_w;
@@ -120,6 +123,7 @@ struct __fehoptions {
 	unsigned int geom_w;
 	unsigned int geom_h;
 	int default_zoom;
+	int zoom_mode;
 	unsigned char adjust_reload;
 
 	unsigned char mode;
@@ -130,6 +134,67 @@ struct __fehoptions {
 	Imlib_Font menu_fn;
 };
 
+struct __fehkey {
+	int keysyms[3];
+	int keystates[3];
+};
+
+struct __fehkb {
+	struct __fehkey menu_close;
+	struct __fehkey menu_parent;
+	struct __fehkey menu_down;
+	struct __fehkey menu_up;
+	struct __fehkey menu_child;
+	struct __fehkey menu_select;
+	struct __fehkey scroll_right;
+	struct __fehkey prev_img;
+	struct __fehkey scroll_left;
+	struct __fehkey next_img;
+	struct __fehkey scroll_up;
+	struct __fehkey scroll_down;
+	struct __fehkey jump_back;
+	struct __fehkey quit;
+	struct __fehkey jump_fwd;
+	struct __fehkey remove;
+	struct __fehkey delete;
+	struct __fehkey jump_first;
+	struct __fehkey jump_last;
+	struct __fehkey action_0;
+	struct __fehkey action_1;
+	struct __fehkey action_2;
+	struct __fehkey action_3;
+	struct __fehkey action_4;
+	struct __fehkey action_5;
+	struct __fehkey action_6;
+	struct __fehkey action_7;
+	struct __fehkey action_8;
+	struct __fehkey action_9;
+	struct __fehkey zoom_in;
+	struct __fehkey zoom_out;
+	struct __fehkey zoom_default;
+	struct __fehkey zoom_fit;
+	struct __fehkey render;
+	struct __fehkey toggle_actions;
+	struct __fehkey toggle_filenames;
+	struct __fehkey toggle_info;
+	struct __fehkey toggle_pointer;
+	struct __fehkey toggle_aliasing;
+	struct __fehkey jump_random;
+	struct __fehkey toggle_caption;
+	struct __fehkey toggle_pause;
+	struct __fehkey reload_image;
+	struct __fehkey save_image;
+	struct __fehkey save_filelist;
+	struct __fehkey size_to_image;
+	struct __fehkey toggle_menu;
+	struct __fehkey close;
+	struct __fehkey orient_1;
+	struct __fehkey orient_3;
+	struct __fehkey toggle_fullscreen;
+	struct __fehkey reload_minus;
+	struct __fehkey reload_plus;
+};
+
 void init_parse_options(int argc, char **argv);
 char *feh_string_normalize(char *str);
 
