@@ -364,7 +364,7 @@ void init_thumbnail_mode(void)
 		if (opt.display) {
 			/* thumb_counter is unsigned, so no need to catch overflows */
 			if (++thumb_counter == opt.thumb_redraw) {
-				winwidget_render_image(winwid, 0, 0);
+				winwidget_render_image(winwid, 0, 1);
 				thumb_counter = 0;
 			}
 			if (!feh_main_iteration(0))
@@ -373,10 +373,10 @@ void init_thumbnail_mode(void)
 	}
 
 	if (thumb_counter != 0)
-		winwidget_render_image(winwid, 0, 0);
+		winwidget_render_image(winwid, 0, 1);
 
 	if (opt.verbose)
-		fprintf(stdout, "\n");
+		putc('\n', stdout);
 
 	if (opt.title_font) {
 		int fw, fh, fx, fy;
@@ -563,19 +563,19 @@ void feh_thumbnail_calculate_geometry(void)
 	if (!opt.limit_w && !opt.limit_h) {
 		if (td.im_bg) {
 			if (opt.verbose)
-				fprintf(stdout,
-					PACKAGE
+				fputs(PACKAGE
 					" - No size restriction specified for index.\n"
 					" You did specify a background however, so the\n"
-					" index size has defaulted to the size of the image\n");
+					" index size has defaulted to the size of the image\n",
+					stdout);
 			opt.limit_w = td.bg_w;
 			opt.limit_h = td.bg_h;
 		} else {
 			if (opt.verbose)
-				fprintf(stdout,
-					PACKAGE
+				fputs(PACKAGE
 					" - No size restriction specified for index.\n"
-					" Using defaults (width limited to 640)\n");
+					" Using defaults (width limited to 640)\n",
+					stdout);
 			opt.limit_w = 640;
 		}
 	}
@@ -901,6 +901,104 @@ int feh_thumbnail_get_generated(Imlib_Image * image, feh_file * file,
 	return (0);
 }
 
+void feh_thumbnail_show_fullsize(feh_file *thumbfile)
+{
+	winwidget thumbwin = NULL;
+	char *s;
+
+	if (!opt.thumb_title)
+		s = thumbfile->name;
+	else
+		s = feh_printf(opt.thumb_title, thumbfile);
+	
+	thumbwin = winwidget_get_first_window_of_type(WIN_TYPE_THUMBNAIL_VIEWER);
+	if (!thumbwin) {
+		thumbwin = winwidget_create_from_file(
+				gib_list_add_front(NULL, thumbfile),
+				s, WIN_TYPE_THUMBNAIL_VIEWER);
+		if (thumbwin)
+			winwidget_show(thumbwin);
+	} else if (FEH_FILE(thumbwin->file->data) != thumbfile) {
+		free(thumbwin->file);
+		thumbwin->file = gib_list_add_front(NULL, thumbfile);
+		winwidget_rename(thumbwin, s);
+		feh_reload_image(thumbwin, 1, 0);
+	}
+}
+
+void feh_thumbnail_select(winwidget winwid, feh_thumbnail *thumbnail)
+{
+	Imlib_Image origwin;
+
+	if (thumbnail == td.selected)
+		return;
+
+	if (thumbnail) {
+		origwin = winwid->im;
+		winwid->im = gib_imlib_clone_image(origwin);
+		gib_imlib_image_fill_rectangle(winwid->im,
+				thumbnail->x, thumbnail->y, thumbnail->w,
+				thumbnail->h, 50, 50, 255, 100);
+		gib_imlib_image_draw_rectangle(winwid->im,
+				thumbnail->x, thumbnail->y, thumbnail->w,
+				thumbnail->h, 255, 255, 255, 255);
+		gib_imlib_image_draw_rectangle(winwid->im,
+				thumbnail->x + 1, thumbnail->y + 1,
+				thumbnail->w - 2, thumbnail->h - 2,
+				0, 0, 0, 255);
+		gib_imlib_image_draw_rectangle(winwid->im,
+				thumbnail->x + 2, thumbnail->y + 2,
+				thumbnail->w - 4, thumbnail->h - 4,
+				255, 255, 255, 255);
+		winwidget_render_image(winwid, 0, 0);
+		gib_imlib_free_image_and_decache(winwid->im);
+		winwid->im = origwin;
+	} else
+		winwidget_render_image(winwid, 0, 0);
+
+	td.selected = thumbnail;
+}
+
+void feh_thumbnail_select_next(winwidget winwid, int jump)
+{
+	gib_list *l, *tmp;
+	int i;
+
+	for (l = thumbnails; l && l->next; l = l->next) {
+		tmp = l;
+		for (i = jump; (i > 0) && tmp->next; i--)
+			tmp = tmp->next;
+		if (tmp->data == td.selected)
+			break;
+	}
+
+	feh_thumbnail_select(winwid, FEH_THUMB(l->data));
+}
+
+void feh_thumbnail_select_prev(winwidget winwid, int jump)
+{
+	gib_list *l;
+	feh_thumbnail *thumb;
+	int i;
+
+	for (l = thumbnails; l; l = l->next) {
+		thumb = FEH_THUMB(l->data);
+		if ((thumb == td.selected) && l->next) {
+			for (i = jump; (i > 0) && l->next; i--)
+				l = l->next;
+			feh_thumbnail_select(winwid, FEH_THUMB(l->data));
+			return;
+		}
+	}
+	feh_thumbnail_select(winwid, FEH_THUMB(thumbnails->data));
+}
+
+inline void feh_thumbnail_show_selected()
+{
+	if (td.selected && td.selected->file)
+		feh_thumbnail_show_fullsize(td.selected->file);
+}
+
 int feh_thumbnail_setup_thumbnail_dir(void)
 {
 	int status = 0;
