@@ -63,6 +63,7 @@ typedef struct thumbmode_data {
 	int cache_thumbnails;    /* use cached thumbnails from ~/.thumbnails */
 	int cache_dim;           /* 128 = 128x128 ("normal"), 256 = 256x256 ("large") */
 	char *cache_dir;         /* "normal"/"large" (.thumbnails/...) */
+	feh_thumbnail *selected;     /* currently selected thumbnail */
 
 } thumbmode_data;
 
@@ -80,6 +81,11 @@ int feh_thumbnail_get_generated(Imlib_Image * image, feh_file * file, char * thu
 char *feh_thumbnail_get_name(char *uri);
 char *feh_thumbnail_get_name_uri(char *name);
 char *feh_thumbnail_get_name_md5(char *uri);
+void feh_thumbnail_show_fullsize(feh_file *thumbfile);
+void feh_thumbnail_select(winwidget winwid, feh_thumbnail *thumbnail);
+void feh_thumbnail_select_next(winwidget winwid, int jump);
+void feh_thumbnail_select_prev(winwidget winwid, int jump);
+void feh_thumbnail_show_selected();
 
 int feh_thumbnail_setup_thumbnail_dir(void);
 
