@@ -1,6 +1,7 @@
 /* winwidget.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -54,9 +55,11 @@ static winwidget winwidget_allocate(void)
 	ret->im = NULL;
 	ret->name = NULL;
 	ret->file = NULL;
+	ret->errstr = NULL;
 	ret->type = WIN_TYPE_UNSET;
 	ret->visible = 0;
 	ret->caption_entry = 0;
+	ret->force_aliasing = opt.force_aliasing;
 
 	/* Zoom stuff */
 	ret->mode = MODE_NORMAL;
@@ -98,7 +101,7 @@ winwidget winwidget_create_from_image(Imlib_Image im, char *name, char type)
 	if (opt.full_screen && (type != WIN_TYPE_THUMBNAIL))
 		ret->full_screen = True;
 	winwidget_create_window(ret, ret->w, ret->h);
-	winwidget_render_image(ret, 1, 1);
+	winwidget_render_image(ret, 1, 0);
 
 	return(ret);
 }
@@ -131,7 +134,7 @@ winwidget winwidget_create_from_file(gib_list * list, char *name, char type)
 		if (opt.full_screen)
 			ret->full_screen = True;
 		winwidget_create_window(ret, ret->w, ret->h);
-		winwidget_render_image(ret, 1, 1);
+		winwidget_render_image(ret, 1, 0);
 	}
 
 	return(ret);
@@ -299,12 +302,21 @@ void winwidget_create_window(winwidget ret, int w, int h)
 void winwidget_update_title(winwidget ret)
 {
 	char *name;
+	Atom prop_name = XInternAtom(disp, "_NET_WM_NAME", False);
+	Atom prop_icon = XInternAtom(disp, "_NET_WM_ICON_NAME", False);
+	Atom prop_utf8 = XInternAtom(disp, "UTF8_STRING", False);
 
 	D(("winwid->name = %s\n", ret->name));
 	name = ret->name ? ret->name : "feh";
 	XStoreName(disp, ret->win, name);
 	XSetIconName(disp, ret->win, name);
-	/* XFlush(disp); */
+
+	XChangeProperty(disp, ret->win, prop_name, prop_utf8, 8,
+			PropModeReplace, (const unsigned char *)name, strlen(name));
+
+	XChangeProperty(disp, ret->win, prop_icon, prop_utf8, 8,
+			PropModeReplace, (const unsigned char *)name, strlen(name));
+
 	return;
 }
 
@@ -361,10 +373,11 @@ void winwidget_setup_pixmaps(winwidget winwid)
 	return;
 }
 
-void winwidget_render_image(winwidget winwid, int resize, int alias)
+void winwidget_render_image(winwidget winwid, int resize, int force_alias)
 {
 	int sx, sy, sw, sh, dx, dy, dw, dh;
 	int calc_w, calc_h;
+	int antialias = 0;
 
 	if (!winwid->full_screen && resize) {
 		winwidget_resize(winwid, winwid->im_w, winwid->im_h);
@@ -377,16 +390,19 @@ void winwidget_render_image(winwidget winwid, int resize, int alias)
 	if (winwid->im_y > winwid->h)
 		winwid->im_y = winwid->h;
 
-	D(("winwidget_render_image resize %d alias %d im %dx%d\n",
-	      resize, alias, winwid->im_w, winwid->im_h));
+	D(("winwidget_render_image resize %d force_alias %d im %dx%d\n",
+	      resize, force_alias, winwid->im_w, winwid->im_h));
 
 	winwidget_setup_pixmaps(winwid);
 
 	if (!winwid->full_screen && opt.scale_down && ((winwid->w < winwid->im_w)
-						       || (winwid->h < winwid->im_h))) {
+						       || (winwid->h < winwid->im_h)) &&
+							  (winwid->old_zoom == 1.0)) {
 		D(("scaling down image %dx%d\n", winwid->w, winwid->h));
 
 		feh_calc_needed_zoom(&(winwid->zoom), winwid->im_w, winwid->im_h, winwid->w, winwid->h);
+		if (resize)
+			 winwidget_resize(winwid, winwid->im_w * winwid->zoom, winwid->im_h * winwid->zoom);
 		D(("after scaling down image %dx%d\n", winwid->w, winwid->h));
 	}
 
@@ -422,7 +438,7 @@ void winwidget_render_image(winwidget winwid, int resize, int alias)
 		smaller = ((winwid->im_w < max_w)
 			   && (winwid->im_h < max_h));
 
-		if (!smaller || opt.auto_zoom) {
+		if (!smaller || opt.zoom_mode) {
 			double ratio = 0.0;
 
 			/* Image is larger than the screen (so wants shrinking), or it's
@@ -448,10 +464,12 @@ void winwidget_render_image(winwidget winwid, int resize, int alias)
 				double old_zoom = winwid->zoom;
 
 				winwid->zoom = 0.01 * opt.default_zoom;
-				if ((winwid->im_h * winwid->zoom) > max_h)
-					winwid->zoom = old_zoom;
-				if ((winwid->im_w * winwid->zoom) > max_w)
-					winwid->zoom = old_zoom;
+				if (winwid->zoom != 1.0) {
+					if ((winwid->im_h * winwid->zoom) > max_h)
+						winwid->zoom = old_zoom;
+					else if ((winwid->im_w * winwid->zoom) > max_w)
+						winwid->zoom = old_zoom;
+				}
 
 				winwid->im_x = ((int)
 						(max_w - (winwid->im_w * winwid->zoom))) >> 1;
@@ -520,17 +538,22 @@ void winwidget_render_image(winwidget winwid, int resize, int alias)
 	D(("sx: %d sy: %d sw: %d sh: %d dx: %d dy: %d dw: %d dh: %d zoom: %f\n",
 	   sx, sy, sw, sh, dx, dy, dw, dh, winwid->zoom));
 
+	if ((winwid->zoom != 1.0) && !force_alias && !winwid->force_aliasing)
+		antialias = 1;
+
 	D(("winwidget_render(): winwid->im_angle = %f\n", winwid->im_angle));
 	if (winwid->has_rotated)
 		gib_imlib_render_image_part_on_drawable_at_size_with_rotation
-		    (winwid->bg_pmap, winwid->im, sx, sy, sw, sh, dx, dy, dw, dh, winwid->im_angle, 1, 1, alias);
+			(winwid->bg_pmap, winwid->im, sx, sy, sw, sh, dx, dy, dw, dh,
+			winwid->im_angle, 1, 1, antialias);
 	else
 		gib_imlib_render_image_part_on_drawable_at_size(winwid->bg_pmap,
 								winwid->im,
 								sx, sy, sw,
 								sh, dx, dy,
 								dw, dh, 1,
-								gib_imlib_image_has_alpha(winwid->im), alias);
+								gib_imlib_image_has_alpha(winwid->im),
+								antialias);
 
 	if (opt.mode == MODE_NORMAL) {
 		if (opt.caption_path)
@@ -539,9 +562,11 @@ void winwidget_render_image(winwidget winwid, int resize, int alias)
 			feh_draw_filename(winwid);
 		if (opt.draw_actions)
 			feh_draw_actions(winwid);
-		if (opt.info_cmd)
+		if (opt.draw_info && opt.info_cmd)
 			feh_draw_info(winwid);
-	} else if ((opt.mode == MODE_ZOOM) && !alias)
+		if (winwid->errstr)
+			feh_draw_errstr(winwid);
+	} else if ((opt.mode == MODE_ZOOM) && !antialias)
 		feh_draw_zoom(winwid);
 
 	XSetWindowBackgroundPixmap(disp, winwid->win, winwid->bg_pmap);
@@ -564,7 +589,7 @@ void winwidget_render_image_cached(winwidget winwid)
 		feh_draw_filename(winwid);
 	if (opt.draw_actions)
 		feh_draw_actions(winwid);
-	if (opt.info_cmd)
+	if (opt.draw_info && opt.info_cmd)
 		feh_draw_info(winwid);
 	XSetWindowBackgroundPixmap(disp, winwid->win, winwid->bg_pmap);
 	XClearWindow(disp, winwid->win);
@@ -576,6 +601,9 @@ double feh_calc_needed_zoom(double *zoom, int orig_w, int orig_h, int dest_w, in
 
 	ratio = ((double) orig_w / orig_h) / ((double) dest_w / dest_h);
 
+	if (opt.zoom_mode == ZOOM_MODE_FILL)
+		ratio = 1.0 / ratio;
+
 	if (ratio > 1.0)
 		*zoom = ((double) dest_w / orig_w);
 	else
@@ -662,10 +690,6 @@ void winwidget_destroy(winwidget winwid)
 	winwidget_destroy_xwin(winwid);
 	if (winwid->name)
 		free(winwid->name);
-	if ((winwid->type == WIN_TYPE_ABOUT) && winwid->file) {
-		feh_file_free(FEH_FILE(winwid->file->data));
-		free(winwid->file);
-	}
 	if (winwid->gc)
 		XFreeGC(disp, winwid->gc);
 	if (winwid->im)
@@ -684,13 +708,13 @@ void winwidget_destroy_all(void)
 	return;
 }
 
-void winwidget_rerender_all(int resize, int alias)
+void winwidget_rerender_all(int resize)
 {
 	int i;
 
 	/* Have to DESCEND the list here, 'cos of the way _unregister works */
 	for (i = window_num - 1; i >= 0; i--)
-		winwidget_render_image(windows[i], resize, alias);
+		winwidget_render_image(windows[i], resize, 0);
 	return;
 }
 
@@ -760,15 +784,17 @@ void winwidget_resize(winwidget winwid, int w, int h)
 			winwid->w = (w > scr->width) ? scr->width : w;
 			winwid->h = (h > scr->height) ? scr->height : h;
 		}
-		/* XResizeWindow(disp, winwid->win, winwid->w, winwid->h); */
-		XGetWindowAttributes(disp, winwid->win, &attributes);
-		XTranslateCoordinates(disp, winwid->win, attributes.root,
-				      -attributes.border_width -
-				      attributes.x,
-				      -attributes.border_width - attributes.y, &tc_x, &tc_y, &ignored_window);
-		winwid->x = tc_x;
-		winwid->y = tc_y;
-		XMoveResizeWindow(disp, winwid->win, tc_x, tc_y, winwid->w, winwid->h);
+		if (winwid->full_screen) {
+			XGetWindowAttributes(disp, winwid->win, &attributes);
+			XTranslateCoordinates(disp, winwid->win, attributes.root,
+						-attributes.border_width -
+						attributes.x,
+						-attributes.border_width - attributes.y, &tc_x, &tc_y, &ignored_window);
+			winwid->x = tc_x;
+			winwid->y = tc_y;
+			XMoveResizeWindow(disp, winwid->win, tc_x, tc_y, winwid->w, winwid->h);
+		} else
+			XResizeWindow(disp, winwid->win, winwid->w, winwid->h);
 
 		winwid->had_resize = 1;
 		XFlush(disp);
@@ -906,6 +932,7 @@ void feh_debug_print_winwid(winwidget w)
 void winwidget_reset_image(winwidget winwid)
 {
 	winwid->zoom = 1.0;
+	winwid->old_zoom = 1.0;
 	winwid->im_x = 0;
 	winwid->im_y = 0;
 	winwid->im_angle = 0.0;
@@ -932,11 +959,11 @@ void winwidget_center_image(winwidget winwid)
 		winwid->im_y = (scr_height - lround(winwid->im_h * winwid->zoom)) >> 1;
 	} else {
 		if (opt.geom_flags & WidthValue)
-			winwid->im_x = (opt.geom_w - lround(winwid->im_w * winwid->zoom)) >> 1;
+			winwid->im_x = ((int)opt.geom_w - lround(winwid->im_w * winwid->zoom)) >> 1;
 		else
 			winwid->im_x = 0;
 		if (opt.geom_flags & HeightValue)
-			winwid->im_y = (opt.geom_h - lround(winwid->im_h * winwid->zoom)) >> 1;
+			winwid->im_y = ((int)opt.geom_h - lround(winwid->im_h * winwid->zoom)) >> 1;
 		else
 			winwid->im_y = 0;
 	}
@@ -980,7 +1007,7 @@ void winwidget_size_to_image(winwidget winwid)
 {
 	winwidget_resize(winwid, winwid->im_w * winwid->zoom, winwid->im_h * winwid->zoom);
 	winwid->im_x = winwid->im_y = 0;
-	winwidget_render_image(winwid, 0, 1);
+	winwidget_render_image(winwid, 0, 0);
 	return;
 }
 
@@ -1043,11 +1070,7 @@ void winwidget_show_menu(winwidget winwid)
 	Window r;
 
 	XQueryPointer(disp, winwid->win, &r, &r, &x, &y, &b, &b, &c);
-	if (winwid->type == WIN_TYPE_ABOUT) {
-		if (!menu_about_win)
-			feh_menu_init_about_win();
-		feh_menu_show_at_xy(menu_about_win, winwid, x, y);
-	} else if (winwid->type == WIN_TYPE_SINGLE) {
+	if (winwid->type == WIN_TYPE_SINGLE) {
 		if (!menu_single_win)
 			feh_menu_init_single_win();
 		feh_menu_show_at_xy(menu_single_win, winwid, x, y);
