@@ -70,7 +70,6 @@ typedef struct _mwmhints {
 
 enum win_type {
 	WIN_TYPE_UNSET, WIN_TYPE_SLIDESHOW, WIN_TYPE_SINGLE,
-	WIN_TYPE_ABOUT,
 	WIN_TYPE_THUMBNAIL, WIN_TYPE_THUMBNAIL_VIEWER
 };
 
@@ -82,6 +81,7 @@ struct __winwidget {
 	int h;
 	int im_w;
 	int im_h;
+	int force_aliasing;
 	double im_angle;
 	enum win_type type;
 	unsigned char had_resize, full_screen;
@@ -92,6 +92,7 @@ struct __winwidget {
 	char *name;
 	gib_list *file;
 	unsigned char visible;
+	char *errstr;
 
 	/* panning, zooming, etc. */
 	unsigned char mode;
@@ -123,14 +124,14 @@ void winwidget_hide(winwidget winwid);
 void winwidget_destroy_all(void);
 void winwidget_free_image(winwidget w);
 void winwidget_center_image(winwidget w);
-void winwidget_render_image(winwidget winwid, int resize, int alias);
+void winwidget_render_image(winwidget winwid, int resize, int force_alias);
 void winwidget_rotate_image(winwidget winid, double angle);
 void winwidget_move(winwidget winwid, int x, int y);
 void winwidget_resize(winwidget winwid, int w, int h);
 void winwidget_setup_pixmaps(winwidget winwid);
 void winwidget_update_title(winwidget ret);
 void winwidget_update_caption(winwidget winwid);
-void winwidget_rerender_all(int resize, int alias);
+void winwidget_rerender_all(int resize);
 void winwidget_destroy_xwin(winwidget winwid);
 
 void winwidget_set_pointer(winwidget winwid, int visible);
